@@ -43,10 +43,16 @@ class PhoneNumber:
 
     def __call__(self, form, field):
         try:
+            # Try parsing number in E.164 format
             p = phonenumbers.parse(field.data)
-            if not phonenumbers.is_valid_number(p):
-                raise ValueError()
-        except (phonenumbers.phonenumberutil.NumberParseException, ValueError):
+        except phonenumbers.phonenumberutil.NumberParseException:
+            try:
+                # Try parsing number for Canada region
+                p = phonenumbers.parse(field.data, "CA")
+            except phonenumbers.phonenumberutil.NumberParseException:
+                raise ValidationError(self.message)
+
+        if not phonenumbers.is_valid_number(p):
             raise ValidationError(self.message)
 
 
@@ -66,8 +72,8 @@ class PostForm(FlaskForm):
     """Ad post form"""
 
     adtype = SelectField('Ad Type', choices=[], validate_choice=False)
-    adtitle = StringField('Ad Title', [InputRequired(), Length(min=8, max=64, message='Title must be between %(min)d and %(max)d characters')])
-    description = TextAreaField('Description', [InputRequired(), Length(min=10, message='Description must be %(min)d or more characters')])
+    adtitle = StringField('Ad Title', [InputRequired(), Length(min=8, max=64, message='Title must be between %(min)d and %(max)d characters')], default="test ad title")
+    description = TextAreaField('Description', [InputRequired(), Length(min=10, message='Description must be %(min)d or more characters')], default="test ad description")
 
     pricetype_choices = [
         ('SPECIFIED_AMOUNT', 'Specified Amount'),  # First element in list must be for a dollar amount
@@ -76,12 +82,12 @@ class PostForm(FlaskForm):
         ('SWAP_TRADE', 'Swap/Trade'),
     ]
     pricetype = SelectField('Price Type', choices=pricetype_choices)
-    price = DecimalField('Price', [RequiredIfFieldEqualTo('pricetype', pricetype_choices[0][0]), NumberRange(min=1), Optional()])
+    price = DecimalField('Price', [RequiredIfFieldEqualTo('pricetype', pricetype_choices[0][0]), NumberRange(min=1), Optional()], default=9999)
 
     loc1 = SelectField('Location', validate_choice=False)
     loc2 = SelectField(validate_choice=False)
     loc3 = SelectField(validate_choice=False)
-    postalcode = StringField('Postal Code', [InputRequired(), PostalCode()])
+    postalcode = StringField('Postal Code', [InputRequired(), PostalCode()], default="A1A1A1")
     # TODO: Implement full address field
     #fulladdress = StringField('Full Address')
     phone = StringField('Phone', [PhoneNumber(), Optional()])
